#pragma once

#include <optional>
#include <vector>

#include <vulkan/vulkan.hpp>

class Swapchain {
public:
    [[nodiscard]]
    static auto choose_extent(
        const vk::Extent2D&               framebuffer_size,
        const vk::SurfaceCapabilitiesKHR& surface_capabilities
    ) -> vk::Extent2D;

    [[nodiscard]]
    static auto create(
        vk::SurfaceKHR      surface,
        vk::PhysicalDevice  physical_device,
        uint32_t            queue_family_index,
        vk::Device          device,
        const vk::Extent2D& framebuffer_size
    ) -> std::optional<Swapchain>;

    [[nodiscard]]
    auto get() const noexcept -> vk::SwapchainKHR;

    [[nodiscard]]
    auto extent() const noexcept -> vk::Extent2D;
    [[nodiscard]]
    auto format() const noexcept -> vk::Format;

private:
    vk::Extent2D           m_extent;
    vk::Format             m_format;
    vk::UniqueSwapchainKHR m_swapchain;

    explicit Swapchain(
        const vk::Extent2D&      extent,
        vk::Format               format,
        vk::UniqueSwapchainKHR&& swapchain
    ) noexcept;
};
